--
--  Copyright (C) 2008, AdaCore
--
with Operations.Binary.Gen_Test.Gen_Suite;
with Operations.Addition.Test;
package Operations.Addition.Test.Suite is new
  Operations.Addition.Test.Gen_Suite ("Operations.Addition");
